package com.dhcc.finance.main.dao;

import java.util.Map;

import org.apache.ibatis.annotations.Param;

public interface CwTjkOutcwVoucherMapper {
	/**
	 * 功能描述   校验：从表的科目是否都在科目关系表存在
	 * @return Long
	 * @author huangpeng
	 * @param map 
	 * @date 2019年7月24日 下午1:41:40
	 * 修改日志
	 */
	String selectExistSubjectCount(Map<String, Object> map);
	/**
	 * 功能描述
	 * @param map
	 * @return int
	 * @author huangpeng
	 * @date 2019年7月26日 下午2:47:05
	 * 修改日志
	 */
	int insertVoucherMainInfo(Map<String, Object> map);
	/**
	 * 功能描述
	 * @param map
	 * @return int
	 * @author huangpeng
	 * @date 2019年7月26日 下午2:47:09
	 * 修改日志
	 */
	int insertDetailSideInfo(Map<String, Object> map);
	/**
	 * 功能描述：删除凭证
	 * @param map
	 * @return int
	 * @author dml
	 * @date 2019年10月19日 下午1:26:07
	 * @修改日志：
	 */
	int deleteForVoucher(Map<String, Object> map);
	/**
	 * 功能描述：删除分录
	 * @param map
	 * @return int
	 * @author dml
	 * @date 2019年10月19日 下午1:26:19
	 * @修改日志：
	 */
	int deleteForDetail(Map<String, Object> map);
	/**
	 * 功能描述：插入辅助项
	 * @param map
	 * @return int
	 * @author dml
	 * @date 2019年10月19日 下午4:05:49
	 * @修改日志：
	 */
	int insertForFreevalue(Map<String, Object> map);
	/**
	 * 功能描述：删除辅助项
	 * @param map
	 * @return int
	 * @author dml
	 * @date 2019年10月19日 下午4:06:07
	 * @修改日志：
	 */
	int deleteForFreevalue(Map<String, Object> map);
	/**
	 * 功能描述：查询是否已跑过
	 * @param pkGlorgbook
	 * @param period
	 * @param beginDate
	 * @param endDate
	 * @return long
	 * @author dml
	 * @date 2020年1月6日 下午9:26:30
	 * @修改日志：
	 */
	long countExistThisBatch(@Param("pkGlorgbook") String pkGlorgbook,@Param("period") String period,
			@Param("beginDate") String beginDate,@Param("endDate") String endDate);
	/**
	 * 功能描述：查询对照关系是否存在非末级科目
	 * @param map
	 * @return String
	 * @author dml
	 * @date 2020年1月9日 上午11:53:29
	 * @修改日志：
	 */
	String selectIsFinalstage(Map<String, Object> map);
}